var restify = require('restify');
var extend = require('extend');
var Busboy = require('busboy');

module.exports = function multipartParser (options) {
  if (!options) {
    options = {};
  }

  // Defaults
  options = extend({
    auto: { // Automatic parsing
      fields: true
    },
    multipart: true, // Multipart content parsing
    urlencoded: true, // Urlencoded content parsing
    encoding: 'utf8', // Default encoding

    // Parser limits (busboy)
    limits: {
      fieldNameSize: 100, // Max field name size (Default: 100 bytes).
      fieldSize: 1024 * 1024, // Max field value size (Default: 1MB)
      fields: undefined, // Max number of non-file fields (Default: Infinity).
      fileSize: undefined, // For multipart forms, the max file size (Default: Infinity).
      files: undefined, // For multipart forms, the max number of file fields (Default: Infinity).
      parts: undefined, // For multipart forms, the max number of parts (fields + files) (Default: Infinity).
      headerPairs: 100 // For multipart forms, the max number of header key=>value pairs to parse Default: 2000 (same as node's http).
    }
  }, options);

  return function middleware (req, res, next) {
    // Allow POST and PUT only
    if ([ 'POST', 'PUT' ].indexOf(req.method) === -1) {
      return next();
    }

    // Ignore non chunked requests
    if (req.contentLength() === 0 && !req.isChunked()) {
      return next();
    }

    // Filter by content type
    var contentType = req.contentType();
    var allowedContentTypes = [ 'multipart/form-data', 'application/x-www-form-urlencoded' ];

    if (allowedContentTypes.indexOf(contentType) === -1) {
      return next();
    }

    if (contentType === allowedContentTypes[0] && !options.multipart) {
      return next();
    } else if (contentType === allowedContentTypes[1] && !options.urlencoded) {
      return next();
    }

    // Init parser
    var body = new Busboy({
      headers: req.headers,
      defCharset: options.encoding,
      limits: options.limits
    });

    var fields = {};

    body.on('file', function onFile (field, file, filename, encoding, mimetype) {
      req.emit('file', field, file, filename, encoding, mimetype);
    });

    body.on('field', function onField (field, value) {
      if (options.auto.fields) {
        if (fields.hasOwnProperty(field)) {
          if (fields[field] instanceof Array) {
            fields[field].push(value);
          } else {
            fields[field] = [ fields[field], value ];
          }
        } else {
          fields[field] = value;
        }
      } else {
        req.emit('field', field, value);
      }
    });

    body.on('finish', function onFinish () {
      if (options.auto.fields && fields) {
        req.params = fields;
        req.emit('fields', fields);
        delete fields;
      }

      req.emit('finish');
    });

    req.pipe(body);
    req.resume();

    return next();
  };
};