var fs = require('fs');
var must = require('must');
var demand = must;
var request = require('supertest');
var restify = require('restify');
var multipartParser = require('../index');

function boundary (prefix) {
  if (!this.boundary) {
    prefix = prefix || 'restify-multipart-';
    this.boundary = prefix + Math.random();
  }

  return this.boundary;
}

function part (parts, name, val, opts) {
  opts = opts || {};

  if (!opts.type) {
    opts.type = 'field';
  }

  parts.push('--' + opts.boundary + '\r\n');

  if (opts.fieldType === 'file') {
    if (opts.filename) {
      parts.push('Content-Disposition: form-data; name="' + name + '"; filename="' + opts.filename + '"\r\n');
    }

    if (opts.contentType) {
      parts.push('Content-Type: ' + opts.contentType + '\r\n');
    }
  } else {
    parts.push('Content-Disposition: form-data; name="' + name + '"\r\n');
  }

  parts.push('\r\n');
  parts.push(val);
  parts.push('\r\n');
}

describe('Restify multipart body parser middleware', function () {
  var server = null;

  before(function (done) {
    server = restify.createServer();

    server.pre(restify.pre.pause());
    server.use(multipartParser({
      auto: {
        fields: true
      }
    }));

    server.put('/multipart', function (req, res, next) {
      var data = {
        fields: {},
        files: []
      };

      req.on('fields', function onFields (fields) {
        data.fields = fields;
      });

      req.on('file', function onFile (field, file, filename, encoding, mimetype) {
        data.files.push({
          field: field,
          filename: filename,
          mimetype: mimetype
        });
      });

      req.on('finish', function onFinish () {
        res.json(200, data);
      })
    });

    server.listen(65001, done);
  });

  it('should parse body content', function (done) {
    var filePath = __filename;

    // Build multipart request
    // We must do it manually because superagent doesn't send content-length header
    var parts = [];

    // Fields
    part(parts, 'single', 'value', { boundary: boundary() });
    part(parts, 'multiple', 'value 1', { boundary: boundary() });
    part(parts, 'multiple', 'value 2', { boundary: boundary() });

    // File
    part(parts, 'file', fs.readFileSync(filePath).toString(), {
      fieldType: 'file',
      contentType: 'text/x-markdown',
      filename: 'file',
      boundary: boundary()
    });

    part(parts, 'file2', fs.readFileSync(filePath).toString(), {
      fieldType: 'file',
      contentType: 'text/plain',
      filename: 'file2',
      boundary: boundary()
    });

    parts.push('--' + boundary() + '--');
    var body = parts.join('');

    request('http://localhost:65001')
      .put('/multipart')
      .set('Content-Type', 'multipart/form-data; boundary=' + boundary())
      .set('Content-Length', body.length)
      .accept('json')
      .send(body)
      .expect(200)
      .end(function (err, res) {
        if (err) {
          throw err;
        }

        var data = res.body;

        demand(data.fields.single).to.eql('value');
        demand(data.fields.multiple).to.eql([ 'value 1', 'value 2' ]);

        demand(data.files[0].field).to.eql('file');
        demand(data.files[0].filename).to.eql('file');
        demand(data.files[0].mimetype).to.eql('text/x-markdown');

        demand(data.files[1].field).to.eql('file2');
        demand(data.files[1].filename).to.eql('file2');
        demand(data.files[1].mimetype).to.eql('text/plain');

        done();
      });
  });
});